/*
 * JavaScript Virtual Keyboard, version 2.2
 *
 * Copyright (C) 2006-2007 Dmitriy Khudorozhkov
 *
 * This software is provided "as-is", without any express or implied warranty.
 * In no event will the author be held liable for any damages arising from the
 * use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software
 *    in a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 *
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 3. This notice may not be removed or altered from any source distribution.
 *
 *  - Dmitriy Khudorozhkov, kh_dmitry2001@mail.ru
 */

function VKeyboard(container_id, callback_ref, create_numpad, font_name,
    font_size, font_color, dead_color, bg_color, key_color,
    sel_item_color, border_color, inactive_border_color,
    inactive_key_color, lang_sel_brd_color) {
    return this._construct(container_id, callback_ref, create_numpad,
        font_name, font_size, font_color, dead_color,
        bg_color, key_color, sel_item_color, border_color,
        inactive_border_color, inactive_key_color,
        lang_sel_brd_color);
}

VKeyboard.prototype = {

    kbArray: [],

    _get_event_source: function(e) {
        var event = e ? e : window.event;
        return event.srcElement ? event.srcElement : event.target;
    },

    _setup_event: function(elem, eventType, handler) {
        return (elem.attachEvent ? elem.attachEvent("on" + eventType, handler) : ((elem.addEventListener) ? elem.addEventListener(eventType, handler, false) : false));
    },

    _detach_event: function(elem, eventType, handler) {
        return (elem.detachEvent ? elem.detachEvent("on" + eventType, handler) : ((elem.removeEventListener) ? elem.removeEventListener(eventType, handler, false) : false));
    },

    _setup_style: function(obj, setEvents, top, left, width, height, position, text_align, line_height, font_size, font_weight, padding_left, padding_right) {
        var os = obj.style;

        if (top) os.top = top;
        if (left) os.left = left;
        if (width) os.width = width;
        if (height) os.height = height;

        if (position) os.position = position;

        if (text_align) os.textAlign = text_align;
        if (line_height) os.lineHeight = line_height;
        if (font_size) os.fontSize = font_size;

        os.fontWeight = (font_weight ? font_weight : "normal");

        if (padding_left) os.paddingLeft = padding_left;
        if (padding_right) os.paddingRight = padding_right;

        if (setEvents) {
            this._setup_event(obj, "selectstart", new Function("event", "return false;"));
            this._setup_event(obj, "mousedown", new Function("event", "if(event.preventDefault) event.preventDefault(); return false;"));
        }
    },

    _setup_key: function(parent, id, top, left, width, height, text_align, line_height, font_size, font_weight, padding_left, padding_right) {
        var exists = document.getElementById(id);

        var key = exists ? exists.parentNode : document.createElement("DIV");
        this._setup_style(key, !exists, top, left, width, height, "absolute");

        var key_sub = exists ? exists : document.createElement("DIV");
        key.appendChild(key_sub);
        parent.appendChild(key);

        this._setup_style(key_sub, !exists, "", "", "", line_height, "relative", text_align, line_height, font_size, font_weight, padding_left, padding_right);
        key_sub.id = id;

        return key_sub;
    },

    _findX: function(obj) { return (obj && obj.parentNode) ? parseFloat(obj.parentNode.offsetLeft) : 0; },

    _findY: function(obj) { return (obj && obj.parentNode) ? parseFloat(obj.parentNode.offsetTop) : 0; },

    _findW: function(obj) { return (obj && obj.parentNode) ? parseFloat(obj.parentNode.offsetWidth) : 0; },

    _findH: function(obj) { return (obj && obj.parentNode) ? parseFloat(obj.parentNode.offsetHeight) : 0; },

    _construct: function(container_id, callback_ref, create_numpad, font_name, font_size, font_color, dead_color, bg_color, key_color, sel_item_color, border_color, inactive_border_color, inactive_key_color, lang_sel_brd_color) {
        var exists = (this.Cntr != undefined),
            ct = exists ? this.Cntr : document.getElementById(container_id);

        var changed = (font_size && (font_size != this.fontsize));

        var ff = ((font_name == "") || (font_name == undefined)) ? (this.fontname ? this.fontname : "") : font_name;
        var fs = font_size ? font_size : (this.fontsize ? this.fontsize : "14px");
        var fc = font_color ? font_color : (this.fontcolor ? this.fontcolor : "#000");
        var dc = dead_color ? dead_color : (this.deadcolor ? this.deadcolor : "#F00");
        var bg = bg_color ? bg_color : (this.bgcolor ? this.bgcolor : "#FFF");
        var kc = key_color ? key_color : (this.keycolor ? this.keycolor : "#FFF");
        var bc = border_color ? border_color : (this.bordercolor ? this.bordercolor : "#777");
        var ibc = inactive_border_color ? inactive_border_color : (this.ibc ? this.ibc : "#CCC");
        var ikc = inactive_key_color ? inactive_key_color : (this.ikc ? this.ikc : "#FFF");
        var lsc = lang_sel_brd_color ? lang_sel_brd_color : (this.lsc ? this.lsc : "#F77");
        var lic = sel_item_color ? sel_item_color : (this.lic ? this.lic : "#DDD");

        this.fontname = ff, this.fontsize = fs, this.fontcolor = fc;
        this.bgcolor = bg;
        this.keycolor = kc;
        this.deadcolor = dc;
        this.bordercolor = bc;
        this.ibc = ibc, this.ikc = ikc, this.lsc = lsc, this.lic = lic;

        this._Callback = ((typeof(callback_ref) == "function") && ((callback_ref.length == 1) || (callback_ref.length == 2))) ? callback_ref : (this._Callback ? this._Callback : null);

        if (!exists) {
            this.Cntr = ct;
            this.Caps = this.Shift = false;

            this.DeadAction = [];
            this.DeadAction[1] = null;
            this.keys = [], this.mod = [], this.pad = [];

            VKeyboard.prototype.kbArray[container_id] = this;

            // Many thanks to Peter-Paul Koch (www.quirksmode.org) for the find-pos-X/find-pos-Y algorithms.
            var initX = 0,
                ct_ = ct;
            if (ct_.offsetParent) {
                while (ct_.offsetParent) {
                    initX += ct_.offsetLeft;
                    ct_ = ct_.offsetParent;
                }
            } else if (ct_.x)
                initX += ct_.x;

            var initY = 0;
            ct_ = ct;
            if (ct_.offsetParent) {
                while (ct_.offsetParent) {
                    initY += ct_.offsetTop;
                    ct_ = ct_.offsetParent;
                }
            } else if (ct_.y)
                initY += ct_.y;
        }

        var kb = exists ? ct.childNodes[0] : document.createElement("DIV");

        if (!exists) {
            ct.appendChild(kb);
            ct.style.display = "block";
            ct.style.position = "absolute";
            ct.style.top = initY + "px", ct.style.left = initX + "px";

            kb.style.position = "relative";
            kb.style.top = "0px", kb.style.left = "0px";
        }

        kb.style.border = "1px solid " + bc;

        var kb_main = exists ? kb.childNodes[0] : document.createElement("DIV"),
            ks = kb_main.style;
        if (!exists) {
            kb.appendChild(kb_main);

            ks.position = "relative";
            ks.width = "1px";
            ks.cursor = "default";
        }

        ks.fontFamily = ff, ks.backgroundColor = bg;

        if (!exists || changed) {
            var mag = parseFloat(fs) / 14.0,
                cell = Math.floor(25.0 * mag),
                dcell = 2 * cell;
            var cp = String(cell) + "px",
                lh = String(cell - 2.0) + "px";

            var prevX = 0,
                prevY = 1,
                prevW = 0,
                prevH = 0;

            // Convenience strings:
            var c = "center",
                n = "normal",
                r = "right",
                l = "left",
                e = "&nbsp;",
                pad = String(4 * mag) + "px";

            // Number row:

            var key;
            for (var i = 0; i < 13; i++) {
                this.keys[i] = key = this._setup_key(kb_main, container_id + "___key" + String(i), "1px", (prevX + prevW + 1) + "px", cp, cp, c, lh, fs);

                prevX = this._findX(key), prevW = this._findW(key);
            }

            prevY = this._findY(key);
            prevH = this._findH(key); // universal key height

            var kb_kbp = this._setup_key(kb_main, container_id + "___kbp", "1px", (prevX + prevW + 1) + "px", (2.96 * cell) + "px", cp, r, lh, fs, n, "", pad);
            kb_kbp.innerHTML = "BackSpace";
            this.mod[0] = kb_kbp;

            // Top row:

            prevX = prevY, prevW = 1.48 * cell + 1, prevY = prevH + 2;

            for (; i < 26; i++) {
                this.keys[i] = key = this._setup_key(kb_main, container_id + "___key" + String(i), prevY + "px", (prevX + prevW + 1) + "px", cp, cp, c, lh, fs);

                prevX = this._findX(key), prevW = this._findW(key);
            }

            this.kbpH = this._findX(kb_kbp) + this._findW(kb_kbp);

            // Home row:

            var kb_caps = this._setup_key(kb_main, container_id + "___caps", (prevY + prevH + 1) + "px", "1px", dcell + "px", cp, l, lh, fs, n, pad);
            kb_caps.innerHTML = "Caps";
            this.mod[1] = kb_caps;

            prevX = this._findX(kb_caps), prevW = this._findW(kb_caps), prevY = this._findY(kb_caps);

            for (; i < 38; i++) {
                this.keys[i] = key = this._setup_key(kb_main, container_id + "___key" + String(i), prevY + "px", (prevX + prevW + 1) + "px", cp, cp, c, lh, fs);

                prevX = this._findX(key), prevW = this._findW(key);
            }

            prevY = this._findY(key);
            var s = prevX + prevW + 1;

            /*var kb_enter = this._setup_key(kb_main, container_id + "___enter_l", prevY + "px", s + "px", (this.kbpH - s) + "px", cp, r, lh, fs, n, "", pad);
            kb_enter.innerHTML = "Enter";
            this.mod[2] = kb_enter;

            s = this._findX(this.keys[25]) + this._findW(this.keys[25]) + 1;

            var kb_enter_top = this._setup_key(kb_main, container_id + "___enter_top", EnterTop + "px", s + "px", (this.kbpH - s) + "px", cp, c, cp, fs);
            kb_enter_top.innerHTML = e;
            kb_enter_top.subst = "Enter"; // Special expando
            this.mod[3] = kb_enter_top;*/

            // Bottom row:

            var kb_shift = this._setup_key(kb_main, container_id + "___shift", (prevY + prevH + 1) + "px", "1px", (2.52 * cell) + "px", cp, l, lh, fs, n, pad);
            kb_shift.innerHTML = "Shift";
            this.mod[2] = kb_shift;

            prevX = this._findX(kb_shift), prevW = this._findW(kb_shift), prevY = this._findY(kb_shift);

            for (; i < 48; i++) {
                this.keys[i] = key = this._setup_key(kb_main, container_id + "___key" + String(i), prevY + "px", (prevX + prevW + 1) + "px", cp, cp, c, lh, fs);

                prevX = this._findX(key), prevW = this._findW(key);
            }

            prevY = this._findY(key);

            var kb_shift_r = this._setup_key(kb_main, container_id + "___shift_r", prevY + "px", (prevX + prevW + 1) + "px", (this._findX(kb_kbp) + this._findW(kb_kbp) - prevX - prevW - 1) + "px", cp, r, lh, fs, n, "", pad);
            kb_shift_r.innerHTML = "Shift";
            this.mod[3] = kb_shift_r;

            prevY = prevY + prevH + 1; //this._findY(kb_lang);

            ks.height = (prevY + prevH + 1) + "px";

            prevY += "px";

            var kb_space = this._setup_key(kb_main, container_id + "___space", prevY, String(4.32 * cell) + "px", (6.28 * cell) + "px", cp, c, lh, fs);
            this.mod[4] = kb_space;

            var w = this.kbpH + 1;

            // Numpad:
            if ((create_numpad == undefined) ? true : create_numpad) {
                var w2 = this._create_numpad(container_id, kb_main);
                if (w2 > w) w = w2;
            }

            kb.style.width = ks.width = w + "px";

            if (!exists) this.CurrentLayout = this.avail_langs[0][0];
        }

        this._refresh_layout();

        return this;
    },

    _create_numpad: function(container_id, parent) {
        var c = "center",
            n = "normal",
            l = "left";
        var fs = this.fontsize,
            bc = this.bordercolor;

        var mag = parseFloat(fs) / 14.0,
            cell = Math.floor(25.0 * mag);
        var dcell = 2 * cell,
            dp = (dcell + 1) + "px",
            dp2 = (dcell - 1) + "px";
        var cp = String(cell) + "px",
            lh = String(Math.floor(cell - 2.0)) + "px";

        var edge = (this.kbpH + cell + 1) + "px";

        var kb_pad_eur = this._setup_key(parent, container_id + "___pad_eur", "1px", edge, cp, cp, c, lh, fs);
        kb_pad_eur.innerHTML = "&#x20AC;";
        this.pad[0] = kb_pad_eur;

        var edge_1 = (this._findX(kb_pad_eur) + this._findW(kb_pad_eur) + 1) + "px";

        var kb_pad_slash = this._setup_key(parent, container_id + "___pad_slash", "1px", edge_1, cp, cp, c, lh, fs);
        kb_pad_slash.innerHTML = "/";
        this.pad[1] = kb_pad_slash;

        var edge_2 = (this._findX(kb_pad_slash) + this._findW(kb_pad_slash) + 1) + "px";

        var kb_pad_star = this._setup_key(parent, container_id + "___pad_star", "1px", edge_2, cp, cp, c, lh, fs);
        kb_pad_star.innerHTML = "*";
        this.pad[2] = kb_pad_star;

        var edge_3 = (this._findX(kb_pad_star) + this._findW(kb_pad_star) + 1) + "px";

        var kb_pad_minus = this._setup_key(parent, container_id + "___pad_minus", "1px", edge_3, cp, cp, c, lh, fs);
        kb_pad_minus.innerHTML = "-";
        this.pad[3] = kb_pad_minus;

        this.kbpM = this._findX(kb_pad_minus) + this._findW(kb_pad_minus) + 1;

        var prevH = this._findH(kb_pad_eur),
            edge_Y = (this._findY(kb_pad_eur) + prevH + 1) + "px";

        var kb_pad_7 = this._setup_key(parent, container_id + "___pad_7", edge_Y, edge, cp, cp, c, lh, fs);
        kb_pad_7.innerHTML = "7";
        this.pad[4] = kb_pad_7;

        var kb_pad_8 = this._setup_key(parent, container_id + "___pad_8", edge_Y, edge_1, cp, cp, c, lh, fs);
        kb_pad_8.innerHTML = "8";
        this.pad[5] = kb_pad_8;

        var kb_pad_9 = this._setup_key(parent, container_id + "___pad_9", edge_Y, edge_2, cp, cp, c, lh, fs);
        kb_pad_9.innerHTML = "9";
        this.pad[6] = kb_pad_9;

        var kb_pad_plus = this._setup_key(parent, container_id + "___pad_plus", edge_Y, edge_3, cp, dp, c, dp2, fs);
        kb_pad_plus.innerHTML = "+";
        this.pad[7] = kb_pad_plus;

        edge_Y = (this._findY(kb_pad_7) + prevH + 1) + "px";

        var kb_pad_4 = this._setup_key(parent, container_id + "___pad_4", edge_Y, edge, cp, cp, c, lh, fs);
        kb_pad_4.innerHTML = "4";
        this.pad[8] = kb_pad_4;

        var kb_pad_5 = this._setup_key(parent, container_id + "___pad_5", edge_Y, edge_1, cp, cp, c, lh, fs);
        kb_pad_5.innerHTML = "5";
        this.pad[9] = kb_pad_5;

        var kb_pad_6 = this._setup_key(parent, container_id + "___pad_6", edge_Y, edge_2, cp, cp, c, lh, fs);
        kb_pad_6.innerHTML = "6";
        this.pad[10] = kb_pad_6;

        edge_Y = (this._findY(kb_pad_4) + prevH + 1) + "px";

        var kb_pad_1 = this._setup_key(parent, container_id + "___pad_1", edge_Y, edge, cp, cp, c, lh, fs);
        kb_pad_1.innerHTML = "1";
        this.pad[11] = kb_pad_1;

        var kb_pad_2 = this._setup_key(parent, container_id + "___pad_2", edge_Y, edge_1, cp, cp, c, lh, fs);
        kb_pad_2.innerHTML = "2";
        this.pad[12] = kb_pad_2;

        var kb_pad_3 = this._setup_key(parent, container_id + "___pad_3", edge_Y, edge_2, cp, cp, c, lh, fs);
        kb_pad_3.innerHTML = "3";
        this.pad[13] = kb_pad_3;

        var kb_pad_enter = this._setup_key(parent, container_id + "___pad_enter", edge_Y, edge_3, cp, dp, c, dp2, parseFloat(fs) * 0.643, n);
        kb_pad_enter.innerHTML = "Enter";
        this.pad[14] = kb_pad_enter;

        edge_Y = (this._findY(kb_pad_1) + prevH + 1) + "px";

        var kb_pad_0 = this._setup_key(parent, container_id + "___pad_0", edge_Y, edge, dp, cp, l, lh, fs, "", 7 * mag + "px");
        kb_pad_0.innerHTML = "0";
        this.pad[15] = kb_pad_0;

        var kb_pad_period = this._setup_key(parent, container_id + "___pad_period", edge_Y, edge_2, cp, cp, c, lh, fs);
        kb_pad_period.innerHTML = ".";
        this.pad[16] = kb_pad_period;

        return this.kbpM;
    },

    _set_key_state: function(key, on, textcolor, bordercolor, bgcolor) {
        if (key) {
            var ks = key.style;
            if (ks) {
                if (textcolor) ks.color = textcolor;
                if (bordercolor) ks.border = "1px solid " + bordercolor;
                if (bgcolor) ks.backgroundColor = bgcolor;
            }

            this._detach_event(key, 'mousedown', this._generic_callback_proc);
            if (on)
                this._setup_event(key, 'mousedown', this._generic_callback_proc);
        }
    },

    _refresh_layout: function() {
        var fc = this.fontcolor;
        var kc = this.keycolor,
            ikc = this.ikc;
        var ibc = this.ibc,
            bc = this.bordercolor,
            lic = this.lic;

        var pre = "VKeyboard.prototype.";

        var layout_name = pre + this.CurrentLayout;
        var arr_type = this.Shift ? "shift" : (this.Caps ? "caps" : "normal");

        var nkeys = this.keys.length;

        var norm_arr;
        eval("norm_arr  = " + layout_name + "_normal");
        var caps_arr;
        eval("caps_arr  = " + layout_name + "_caps");
        var shift_arr;
        eval("shift_arr = " + layout_name + "_shift");

        var dead_arr;
        eval("dead_arr = " + (this.DeadAction[1] ? pre + this.DeadAction[1] : "null"));

        var bcaps = (caps_arr && (caps_arr.length == nkeys));
        var bshift = (shift_arr && (shift_arr.length == nkeys));
        var caps = this.mod[1],
            shift = this.mod[2],
            shift_r = this.mod[3];

        if (bshift) {
            this._set_key_state(shift, true, fc, bc, this.Shift ? lic : kc);
            this._set_key_state(shift_r, true, fc, bc, this.Shift ? lic : kc);
        } else {
            this._set_key_state(shift, false, ibc, ibc, ikc);
            this._set_key_state(shift_r, false, ibc, ibc, ikc);

            if (arr_type == "shift") {
                arr_type = "normal";
                this.Shift = false;
            }
        }

        if (bcaps)
            this._set_key_state(caps, true, fc, bc, this.Caps ? lic : kc);
        else {
            this._set_key_state(caps, false, ibc, ibc, ikc);

            this.Caps = false;
            if (arr_type == "caps") arr_type = "normal";
        }

        var arr_cur;
        switch (arr_type) {
            case "caps":
                arr_cur = caps_arr;
                break;
            case "shift":
                arr_cur = shift_arr;
                break;
            default:
                arr_cur = norm_arr;
        }

        var i = nkeys;
        while (--i >= 0) {
            var key = this.keys[i],
                key_val = arr_cur[i];
            if (!key_val) key_val = "";

            if (this.Shift && this.Caps) {
                var key_nrm = norm_arr[i],
                    key_cps = caps_arr[i],
                    key_shf = shift_arr[i];

                if ((key_cps == key_shf) && (key_nrm != key_cps)) key_val = key_nrm;
            }

            if (typeof(key_val) == "object") {
                key.innerHTML = key_val[0], key.dead = key_val[1];

                this._set_key_state(key, true, this.deadcolor, bc, (this.DeadAction[0] == key_val[0] ? lic : kc));
            } else {
                key.dead = null;

                var block = false;

                if (key_val != "") {
                    if (dead_arr) {
                        for (var j = 0; j < dead_arr.length; j++) { var dk = dead_arr[j]; if (dk[0] == key_val) { key_val = dk[1]; break; } };

                        if (j == dead_arr.length) block = true;
                    }

                    key.innerHTML = key_val;

                    if (block)
                        this._set_key_state(key, false, ibc, ibc, ikc);
                    else
                        this._set_key_state(key, true, fc, bc, kc);
                } else {
                    key.innerHTML = "&nbsp;";
                    this._set_key_state(key, false, ibc, ibc, ikc);
                }
            }
        }

        i = this.mod.length;
        while (--i >= 0) {
            var key = this.mod[i];

            switch (i) {
                case 1:
                case 2:
                case 3:
                    break;

                case 4:
                    key.innerHTML = this.DeadAction[1] ? this.DeadAction[0] : "&nbsp;";

                default:
                    if (this.DeadAction[1] && (i != 4))
                        this._set_key_state(key, false, ibc, ibc, ikc);
                    else
                        this._set_key_state(key, true, fc, bc, kc);
            }
        }

        i = this.pad.length;
        while (--i >= 0) {
            key = this.pad[i];

            if (this.DeadAction[1])
                this._set_key_state(key, false, ibc, ibc, ikc);
            else
                this._set_key_state(key, true, fc, bc, kc);
        }
    },

    _generic_callback_proc: function(event) {
        var pr = VKeyboard.prototype;

        var in_el = pr._get_event_source(event);
        var container_id = in_el.id.substring(0, in_el.id.indexOf("___"));
        var vkboard = pr.kbArray[container_id];

        var val = in_el.subst ? in_el.subst : in_el.innerHTML;
        if (!val) return;

        var r = false;
        switch (val) {
            case "Caps":
                r = true;
                vkboard.Caps = !vkboard.Caps;
                break;
            case "Shift":
                r = true;
                vkboard.Shift = !vkboard.Shift;
                break;

            case "&nbsp;":
                val = " ";
                break;
            case "&quot;":
                val = "\"";
                break;
            case "&lt;":
                val = "<";
                break;
            case "&gt;":
                val = ">";
                break;
            case "&amp;":
                val = "&";
                break;
        }
        if (r) { vkboard._refresh_layout(); return; }

        if (in_el.dead) {
            if (in_el.dead == vkboard.DeadAction[1]) { val = "";
                vkboard.DeadAction[0] = vkboard.DeadAction[1] = null; } else { vkboard.DeadAction[0] = val;
                vkboard.DeadAction[1] = in_el.dead; }

            vkboard._refresh_layout();
            return;
        } else {
            if (vkboard.DeadAction[1]) { vkboard.DeadAction[0] = vkboard.DeadAction[1] = null;
                r = true; }

            if (vkboard.Shift || r) {
                vkboard.Shift = false;
                vkboard._refresh_layout();
            }
        }

        if (vkboard._Callback) vkboard._Callback(val, vkboard.Cntr.id);
    },

    SetParameters: function() {
        var l = arguments.length;
        if (!l || (l % 2 != 0)) return false;

        var p0, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11;

        while (--l > 0) {
            var value = arguments[l];

            switch (arguments[l - 1]) {
                case "callback":
                    p0 = ((typeof(value) == "function") && ((value.length == 1) || (value.length == 2))) ? value : this._Callback;
                    break;

                case "font-name":
                    p1 = value;
                    break;
                case "font-size":
                    p2 = value;
                    break;
                case "font-color":
                    p3 = value;
                    break;
                case "dead-color":
                    p4 = value;
                    break;
                case "base-color":
                    p5 = value;
                    break;
                case "key-color":
                    p6 = value;
                    break;

                case "selection-color":
                    p7 = value;
                    break;
                case "border-color":
                    p8 = value;
                    break;

                case "inactive-border-color":
                    p9 = value;
                    break;
                case "inactive-key-color":
                    p10 = value;
                    break;
                case "lang-cell-color":
                    p11 = value;
                    break;

                default:
                    break;
            }

            l -= 1;
        }

        this._construct(this.Cntr.id, p0, (this.pad.length != 0), p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11);

        return true;
    },

    Show: function(value) {
        var ct = this.Cntr.style;

        ct.display = ((value == undefined) || (value == true)) ? "block" : ((value == false) ? "none" : ct.display);
    },

    ShowNumpad: function(value) {
        var sh = ((value == undefined) || (value == true)) ? "block" : ((value == false) ? "none" : null);
        if (!sh) return;

        var kb = this.Cntr.childNodes[0];

        var i = this.pad.length;
        if (i) {
            while (--i >= 0)
                this.pad[i].parentNode.style.display = sh;

            kb.style.width = kb.childNodes[0].style.width = (sh == "none") ? (this.kbpH + 1) + "px" : this.kbpM + "px";
        } else {
            if (sh == "block") {
                kb.style.width = kb.childNodes[0].style.width = this._create_numpad(this.Cntr.id, kb.childNodes[0]);
                this._refresh_layout();
            }
        }
    },

    // Layout info:

    avail_langs: [
        ["Emid", "Emid"]
    ],

    // Emid:

    Emid_normal: ["&#x0060;", "&#x0031;", "&#x0032;", "&#x0033;", "&#x0034;", "&#x0035;", "&#x0036;", "&#x0037;", "&#x0038;", "&#x0039;", "&#x0030;", "&#x002D;", "&#x003D;",
        "&#x0071;", "&#x0077;", "&#x0065;", "&#x0072;", "&#x0074;", "&#x0079;", "&#x0075;", "&#x0069;", "&#x006F;", "&#x0070;", "&#x005B;", "&#x005D;", "&#x005C;",
        "&#x0061;", "&#x0073;", "&#x0064;", "&#x0066;", "&#x0067;", "&#x0068;", "&#x006A;", "&#x006B;", "&#x006C;", "&#x003B;", "&#x0027;", ,
        "&#x007A;", "&#x0078;", "&#x0063;", "&#x0076;", "&#x0062;", "&#x006E;", "&#x006D;", "&#x002C;", "&#x002E;", "&#x002F;"
    ],

    Emid_caps: ["&#x0060;", "&#x0031;", "&#x0032;", "&#x0033;", "&#x0034;", "&#x0035;", "&#x0036;", "&#x0037;", "&#x0038;", "&#x0039;", "&#x0030;", "&#x002D;", "&#x003D;",
        "&#x0051;", "&#x0057;", "&#x0045;", "&#x0052;", "&#x0054;", "&#x0059;", "&#x0055;", "&#x0049;", "&#x004F;", "&#x0050;", "&#x005B;", "&#x005D;", "&#x005C;",
        "&#x0041;", "&#x0053;", "&#x0044;", "&#x0046;", "&#x0047;", "&#x0048;", "&#x004A;", "&#x004B;", "&#x004C;", "&#x003B;", "&#x0027;", ,
        "&#x005A;", "&#x0058;", "&#x0043;", "&#x0056;", "&#x0042;", "&#x004E;", "&#x004D;", "&#x002C;", "&#x002E;", "&#x002F;"
    ],

    Emid_shift: ["&#x007E;", "&#x0021;", "&#x0040;", "&#x0023;", "&#x0024;", "&#x0025;", "&#x005E;", "&#x0026;", "&#x002A;", "&#x0028;", "&#x0029;", "&#x005F;", "&#x002B;",
        "&#x0051;", "&#x0057;", "&#x0045;", "&#x0052;", "&#x0054;", "&#x0059;", "&#x0055;", "&#x0049;", "&#x004F;", "&#x0050;", "&#x007B;", "&#x007D;", "&#x007C;",
        "&#x0041;", "&#x0053;", "&#x0044;", "&#x0046;", "&#x0047;", "&#x0048;", "&#x004A;", "&#x004B;", "&#x004C;", "&#x003A;", "&#x0022;", ,
        "&#x005A;", "&#x0058;", "&#x0043;", "&#x0056;", "&#x0042;", "&#x004E;", "&#x004D;", "&#x003C;", "&#x003E;", "&#x003F;"
    ]
};